from pdm.backend.hooks import Context

from pdm_una.include import force_include
from pdm_una.meta import add_dependencies


def pdm_build_initialize(context: Context):
    add_dependencies(context)
    force_include(context)



from pathlib import Path

from pdm.backend.hooks import Context

from pdm_una import util


def add_dependencies(context: Context):
    """
    Inject needed third-party dependencies into project.dependencies.

    This hook is only needed for Packages style una workspaces.
    Modules style should have all dependencies specified in pyproject.toml.
    """
    print("una: Injecting transitive external dependencies")
    metadata = context.config.metadata
    path = context.root

    conf = util.load_conf(path)
    name: str = conf["project"]["name"]

    root_path = path.parents[1]
    style = util.get_workspace_style(root_path)
    if style == "modules":
        # in Hatch, we raise here (should we?)
        # but in PDM the hooks are not configured separately
        # so just return
        return

    try:
        int_deps: dict[str, str] = conf["tool"]["una"]["libs"]
    except KeyError as e:
        raise KeyError(
            f"App/project '{name}' is missing '[tool.una.libs]' in pyproject.toml"
        ) from e

    project_deps: list[str] = metadata.get("dependencies", [])
    project_deps = [d.strip().replace(" ", "") for d in project_deps]

    add_deps: list[str] = []
    for dep_path in int_deps:
        # In builds that do src -> sdist -> wheel, the needed pyproject.toml files
        # will have been copied into the sdist so they're available for the wheel build.
        # Here we check for both in order.
        dep_project_path = Path(dep_path).parents[1]
        extra_path = util.EXTRA_PYPROJ / Path(dep_path).name
        if dep_project_path.exists():
            use_path = dep_project_path
        elif extra_path.exists():
            use_path = extra_path
        else:
            raise ValueError(f"Could not find internal dependency at '{dep_path}'")

        # load all third-party dependencies from this internal dependency into the
        # project.dependencies table
        dep_conf = util.load_conf(use_path)
        try:
            dep_deps: list[str] = dep_conf["project"]["dependencies"]
        except KeyError as e:
            raise KeyError(f"No project.dependcies table for '{use_path}'")
        dep_deps = [d.strip().replace(" ", "") for d in dep_deps]
        add_deps.extend(dep_deps)

    metadata["dependencies"] = list(set(project_deps + add_deps))



import shutil
import tomllib
from pathlib import Path
from typing import Any, Literal, TypeAlias

PYPROJ = "pyproject.toml"
EXTRA_PYPROJ = Path("_extra_pyproj")
ROOT_PYPROJ_SUBDIR = Path("_root")


def load_conf(path: Path) -> dict[str, Any]:
    with (path / PYPROJ).open("rb") as fp:
        return tomllib.load(fp)


Style: TypeAlias = Literal["packages", "modules"]


def get_workspace_style(root_path: Path) -> Style:
    """
    Get the root workspace style

    Param `path` should be the path to the ap/project,
    NOT to the root workspace.
    """
    # In builds that do src -> sdist -> wheel, the root pyproject.toml file will
    # have been copied into the sdist so available for the wheel build.
    # Here we check for both in order.
    extra_root_path = EXTRA_PYPROJ / ROOT_PYPROJ_SUBDIR
    if (root_path / PYPROJ).exists():
        use_root_path = root_path
    elif (extra_root_path / PYPROJ).exists():
        use_root_path = extra_root_path
    else:
        raise ValueError("No root pyproject to determine workspace style")
    root_conf = load_conf(use_root_path)
    try:
        style: Style = root_conf["tool"]["una"]["style"]
        return style
    except KeyError as e:
        raise KeyError(
            "Root workspace pyproject.toml needs '[tool.una]' with style specified"
        ) from e


def copy_file(src: Path, dst: Path) -> Path:
    dst.parents[0].mkdir(parents=True, exist_ok=True)
    return shutil.copyfile(src, dst)


def copy_tree(src: Path, dst: Path) -> Path:
    ignore = shutil.ignore_patterns(
        "*.pyc",
        "__pycache__",
        ".venv",
        "__pypackages__",
        ".mypy_cache",
        ".pytest_cache",
        "node_modules",
        ".git",
    )

    res: Path = shutil.copytree(  # return might not actually be a Path
        src,
        dst,
        ignore=ignore,
        dirs_exist_ok=True,
    )
    return Path(res)
